/*
 *  gnome2wmaker.c:	GNOME menu output in Window Maker format
 *
 *  Written by:		Ullrich Hafner
 *  
 *  Copyright (C) 1999 Ullrich Hafner <hafner@bigfoot.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

/*
 *  $Date: 1999/02/09 21:57:14 $
 *  $Author: hafner $
 *  $Revision: 1.1 $
 *  $State: Exp $
 */

#include <stdio.h>
#include <dirent.h>
#include <gnome.h>
#include <proplist.h>

#define strcaseeq(s1, s2)	(g_strcasecmp ((s1), (s2)) == 0)
#define streq(str1, str2)	(strcmp ((str1), (str2)) == 0)

/*******************************************************************************

				prototypes
  
*******************************************************************************/

static GList *
get_order_of_dir (const gchar *file);
static void
traverse_gnome_menu (const gchar *filename, int level);

/*******************************************************************************

				public code
  
*******************************************************************************/

int
main (int argc, char **argv)
{
   char *system_apps;

   gnome_init ("gnome2wmaker", VERSION, argc, argv);
   
   system_apps = gnome_unconditional_datadir_file ("gnome/apps");
      
   if (!g_file_exists (system_apps))
   {
      fprintf (stderr, "Can't find GNOME system dir `%s'.\n", system_apps);
      return 1;
   }

   traverse_gnome_menu (system_apps, 0);

   return 0;
}

/*******************************************************************************

				private code
  
*******************************************************************************/

static GList *
get_order_of_dir (const gchar *dirname)
/*
 *  Generates ordered list if menu items if the file 'dirname/.order' exists.
 *
 *  Return value:
 *	list of filenames
 */
{
   char *orderfile = g_concat_dir_and_file (dirname, ".order");
   FILE *order;
   
   if ((order = fopen (orderfile, "r")))
   {
      GList *list = NULL;
      gchar buffer [256];

      while (fgets (buffer, 255, order) != NULL)
      {
	 char *bptr = strchr (buffer,'\n');
	 if (bptr)
	    *bptr = '\0';
	 if (strlen (buffer) > 0)
	    list = g_list_append (list, g_strdup (buffer));
      }
      fclose (order);
      g_free (orderfile);

      return list;
   }
   else
   {
      g_free (orderfile);
      
      return NULL;
   }
}

static void
traverse_gnome_menu (const gchar *filename, int level)
/*
 *  Recursivly traverse GNOME system menu directory structure.
 *
 *  No return value.
 */
{
   DIR *dir;
    
   if ((dir = opendir (filename)))
   {
      GnomeDesktopEntry *dentry;
      GList	  *order;
      const gchar *title;
      gchar	  *dirname = g_concat_dir_and_file (filename, ".directory");
      
      dentry = gnome_desktop_entry_load_unconditional (dirname);
      g_free (dirname);

      if (level > 0)
	 title = dentry ? dentry->name : filename + g_filename_index (filename);
      else if (level < 0)
      {
	 title = "Local";
	 level = 1;
      }
      else
	 title = "GNOME";
      
      fprintf (stdout, "%-*s\"%s\" MENU\n", level * 2, "", title);
      
      if ((order = get_order_of_dir (filename)))
      {
	 unsigned n;

	 for (n = 0; n < g_list_length (order); n++)
	 {
	    gchar *fullname = g_concat_dir_and_file (filename,
						     g_list_nth (order, n)->data);
	    traverse_gnome_menu (fullname, level + 1);
	    g_free (fullname);
	 }

	 g_list_foreach (order, (GFunc) g_free, NULL);
	 g_list_free (order);
      }
      else
      {
	 struct dirent *file;

	 while ((file = readdir (dir)))
	 {
	    if (streq (file->d_name, ".") || streq (file->d_name, ".."))
	       continue;
	    {
	       gchar *fullname = g_concat_dir_and_file (filename, file->d_name);

	       traverse_gnome_menu (fullname, level + 1);
	       g_free (fullname);
	    }
	 }
      }
      
      {
	 static gint user = 0;
	 
	 if (!level && !user)
	 {
	 
	    char *user_apps_dir = gnome_util_home_file ("apps");
	    if (g_file_exists (user_apps_dir))
	       traverse_gnome_menu (user_apps_dir, -1);
	    user = 1;
	 }
      }
      
      fprintf (stdout, "%-*s\"%s\" END\n", level * 2, "", title);

      closedir (dir);
      if (dentry)
	 gnome_desktop_entry_free (dentry);
   }
   else
   {
      GnomeDesktopEntry *dentry;
      
      dentry = gnome_desktop_entry_load_unconditional (filename);
      if (dentry && dentry->type && strcaseeq (dentry->type, "application")
	  && dentry->exec_length > 0)
      {
	 unsigned n;
	 
	 fprintf (stdout, "%-*s\"%s\" EXEC", level * 2, "", dentry->name);
	 for (n = 0; n < dentry->exec_length; n++)
	    fprintf (stdout, " %s", dentry->exec [n]);
	 fprintf (stdout, "\n");
      }
      if (dentry)
	 gnome_desktop_entry_free (dentry);
   }
}
